/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package org.jdtaus.banking.dtaus.swing;

import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Toolkit;
import java.util.Locale;
import javax.swing.JDialog;
import javax.swing.SwingUtilities;
import org.jdtaus.core.container.ContainerFactory;
import org.jdtaus.core.container.ContainerInitializer;
import org.jdtaus.core.container.ContextFactory;
import org.jdtaus.core.container.ContextInitializer;
import org.jdtaus.core.container.Implementation;
import org.jdtaus.core.container.ModelFactory;
import org.jdtaus.core.container.Properties;
import org.jdtaus.core.container.Property;
import org.jdtaus.editor.Editor;
import org.jdtaus.editor.Preferences;

/**
 * Dialog for editing various aspects of the application.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: ConfigurationDialog.java 3516 2007-05-16 10:09:13Z schulte2005 $
 */
public class ConfigurationDialog extends JDialog implements ContainerInitializer
{
    //--Implementation----------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /** Meta-data describing the implementation. */
    private static final Implementation META =
        ModelFactory.getModel().getModules().
        getImplementation(ConfigurationDialog.class.getName());

    //----------------------------------------------------------Implementation--
    //--Constructors------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /**
     * Initializes the properties of the instance.
     *
     * @param meta the property values to initialize the instance with.
     *
     * @throws NullPointerException if {@code meta} is {@code null}.
     */
    protected void initializeProperties(final Properties meta)
    {
        Property p;

        if(meta == null)
        {
            throw new NullPointerException("meta");
        }

    }

    //------------------------------------------------------------Constructors--
    //--ContainerInitializer----------------------------------------------------

    /** Initializes the instance centering the dialog on the screen. */
    public void initialize()
    {
        this.assertValidProperties();

        final Dimension screenSize = Toolkit.getDefaultToolkit().
            getScreenSize();

        this.initComponents();

        this.setLocation(screenSize.width / 2 - this.getWidth() / 2,
            screenSize.height / 2 - this.getHeight() / 2);

    }

    //----------------------------------------------------ContainerInitializer--
    //--Dependencies------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /** Configured <code>Application</code> implementation. */
    private transient Editor _dependency0;

    /**
     * Gets the configured <code>Application</code> implementation.
     *
     * @return the configured <code>Application</code> implementation.
     */
    private Editor getApplication()
    {
        Editor ret = null;
        if(this._dependency0 != null)
        {
            ret = this._dependency0;
        }
        else
        {
            ret = (Editor) ContainerFactory.getContainer().
                getDependency(ConfigurationDialog.class,
                "Application");

            if(ModelFactory.getModel().getModules().
                getImplementation(ConfigurationDialog.class.getName()).
                getDependencies().getDependency("Application").
                isBound())
            {
                this._dependency0 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }

    //------------------------------------------------------------Dependencies--
    //--Properties--------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.


    //--------------------------------------------------------------Properties--
    //--ConfigurationDialog-----------------------------------------------------

    /** A return status code - returned if Cancel button has been pressed */
    public static final int RET_CANCEL = 0;

    /** A return status code - returned if OK button has been pressed */
    public static final int RET_OK = 1;

    /**
     * Creates new {@code ConfigurationDialog} instance.
     *
     * @param owner the frame owning the dialog.
     * @param modal {@code true} for a modal dialog; {@code false} else.
     */
    public ConfigurationDialog(final Frame owner, final boolean modal)
    {
        super(owner, modal);
        this.initialize();
    }

    /**
     * Gets the return status of the dialog.
     *
     * @return the return status of this dialog - one of {@code RET_OK} or
     * {@code RET_CANCEL}.
     */
    public int getReturnStatus()
    {
        return this.returnStatus;
    }

    /**
     * Gets the preferences displayed by the dialog.
     *
     * @return the preferences displayed by the dialog.
     */
    public Preferences getPreferences()
    {
        final Preferences prefs = new Preferences();
        prefs.setBankcodeValidationDisabled(
            this.validateBankcodesCheckBox.isSelected());

        prefs.setFileDeletionConfirmationDisabled(
            this.confirmHeaderDeletionCheckBox.isSelected());

        prefs.setPaymentDeletionConfirmationDisabled(
            this.confirmPaymentDeletionCheckBox.isSelected());

        prefs.setOverwriteConfirmationDisabled(
            this.confirmOverwriteCheckBox.isSelected());

        return prefs;
    }

    /**
     * Sets the preferences to be displayed by the dialog.
     *
     * @param prefs the preferences to be displayed by the dialog.
     *
     * @throws NullPointerException if {@code prefs} is {@code null}.
     */
    public void setPreferences(final Preferences prefs)
    {
        if(prefs == null)
        {
            throw new NullPointerException("prefs");
        }

        final Runnable runnable = new Runnable()
        {
            public void run()
            {
                validateBankcodesCheckBox.setSelected(
                    prefs.isBankcodeValidationDisabled());

                confirmHeaderDeletionCheckBox.setSelected(
                    prefs.isFileDeletionConfirmationDisabled());

                confirmPaymentDeletionCheckBox.setSelected(
                    prefs.isPaymentDeletionConfirmationDisabled());

                confirmOverwriteCheckBox.setSelected(
                    prefs.isOverwriteConfirmationDisabled());

            }
        };

        if(!SwingUtilities.isEventDispatchThread())
        {
            SwingUtilities.invokeLater(runnable);
        }
        else
        {
            runnable.run();
        }
    }

    /**
     * Flag indicating if the dialog is locked for any interaction.
     *
     * @return {@code true} if the dialog is locked for any interaction;
     * {@code false} if the dialog is not locked.
     */
    public boolean isLocked()
    {
        return this.locked;
    }

    /**
     * Setter for property {@code locked}.
     *
     * @param locked {@code true} if the dialog should be locked for any
     * interaction; {@code false} to unlock the dialog.
     */
    public void setLocked(final boolean locked)
    {
        final Runnable runnable = new Runnable()
        {
            public void run()
            {
                setCursor(Cursor.getPredefinedCursor(locked ?
                    Cursor.WAIT_CURSOR : Cursor.DEFAULT_CURSOR));

                cancelButton.setEnabled(!locked);
                confirmHeaderDeletionCheckBox.setEnabled(!locked);
                confirmOverwriteCheckBox.setEnabled(!locked);
                confirmPaymentDeletionCheckBox.setEnabled(!locked);
                okButton.setEnabled(!locked);

                firePropertyChange("locked",
                    ConfigurationDialog.this.locked, locked);

                ConfigurationDialog.this.locked = locked;
            }
        };

        if(!SwingUtilities.isEventDispatchThread())
        {
            SwingUtilities.invokeLater(runnable);
        }
        else
        {
            runnable.run();
        }
    }

    /**
     * Checks configured properties.
     *
     * @throws PropertyException for illegal property values.
     */
    protected void assertValidProperties()
    {}

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents()
    {
        okButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();
        jPanel1 = new javax.swing.JPanel();
        validateBankcodesCheckBox = new javax.swing.JCheckBox();
        confirmHeaderDeletionCheckBox = new javax.swing.JCheckBox();
        confirmPaymentDeletionCheckBox = new javax.swing.JCheckBox();
        confirmOverwriteCheckBox = new javax.swing.JCheckBox();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle(ConfigurationDialogBundle.getTitleText(Locale.getDefault()));
        addWindowListener(new java.awt.event.WindowAdapter()
        {
            public void windowClosing(java.awt.event.WindowEvent evt)
            {
                closeDialog(evt);
            }
        });

        okButton.setText(ConfigurationDialogBundle.getOkButtonText(Locale.getDefault()));
        okButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                okButtonActionPerformed(evt);
            }
        });

        cancelButton.setText(ConfigurationDialogBundle.getCancelButtonText(Locale.getDefault()));
        cancelButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                cancelButtonActionPerformed(evt);
            }
        });

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder(ConfigurationDialogBundle.getApplicationPanelTitleText(Locale.getDefault())));
        validateBankcodesCheckBox.setText(ConfigurationDialogBundle.getValidateBankcodesCheckboxText(Locale.getDefault()));

        confirmHeaderDeletionCheckBox.setText(ConfigurationDialogBundle.getConfirmHeaderDeletionCheckboxText(Locale.getDefault()));

        confirmPaymentDeletionCheckBox.setText(ConfigurationDialogBundle.getConfirmTransactionDeletionCheckboxText(Locale.getDefault()));

        confirmOverwriteCheckBox.setText(ConfigurationDialogBundle.getConfirmOverwriteCheckboxText(Locale.getDefault()));

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(confirmOverwriteCheckBox, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 372, Short.MAX_VALUE)
                    .add(confirmPaymentDeletionCheckBox, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 372, Short.MAX_VALUE)
                    .add(confirmHeaderDeletionCheckBox, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 372, Short.MAX_VALUE)
                    .add(validateBankcodesCheckBox, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 372, Short.MAX_VALUE))
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(validateBankcodesCheckBox)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(confirmHeaderDeletionCheckBox)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(confirmPaymentDeletionCheckBox)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(confirmOverwriteCheckBox)
                .addContainerGap(131, Short.MAX_VALUE))
        );

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                        .add(okButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 67, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(cancelButton))
                    .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );

        layout.linkSize(new java.awt.Component[] {cancelButton, okButton}, org.jdesktop.layout.GroupLayout.HORIZONTAL);

        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(cancelButton)
                    .add(okButton))
                .addContainerGap())
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void okButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_okButtonActionPerformed
    {
        this.getApplication().setPreferences(this.getPreferences());
        this.doClose(RET_OK);
    }//GEN-LAST:event_okButtonActionPerformed

    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_cancelButtonActionPerformed
    {
        this.doClose(RET_CANCEL);
    }//GEN-LAST:event_cancelButtonActionPerformed

    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt)//GEN-FIRST:event_closeDialog
    {
        if(!this.isLocked())
        {
            this.doClose(RET_CANCEL);
        }
    }//GEN-LAST:event_closeDialog

    private void doClose(final int retStatus)
    {
        this.returnStatus = retStatus;
        this.setVisible(false);
        this.dispose();
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton cancelButton;
    private javax.swing.JCheckBox confirmHeaderDeletionCheckBox;
    private javax.swing.JCheckBox confirmOverwriteCheckBox;
    private javax.swing.JCheckBox confirmPaymentDeletionCheckBox;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JButton okButton;
    private javax.swing.JCheckBox validateBankcodesCheckBox;
    // End of variables declaration//GEN-END:variables

    private int returnStatus = RET_CANCEL;

    /**
     * Flag indicating that the user interface is locked.
     * @serial
     */
    private boolean locked;

    //-----------------------------------------------------ConfigurationDialog--
}
