/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package org.jdtaus.editor.spi;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Currency;
import java.util.Date;
import java.util.List;
import org.jdtaus.banking.AlphaNumericText27;
import org.jdtaus.banking.Bankleitzahl;
import org.jdtaus.banking.BankleitzahlenVerzeichnis;
import org.jdtaus.banking.Referenznummer10;
import org.jdtaus.banking.Referenznummer11;
import org.jdtaus.banking.Textschluessel;
import org.jdtaus.banking.dtaus.Header;
import org.jdtaus.banking.dtaus.IllegalHeaderException;
import org.jdtaus.banking.dtaus.IllegalTransactionException;
import org.jdtaus.banking.dtaus.LogicalFile;
import org.jdtaus.banking.dtaus.LogicalFileType;
import org.jdtaus.banking.dtaus.PhysicalFile;
import org.jdtaus.banking.dtaus.PhysicalFileException;
import org.jdtaus.banking.dtaus.PhysicalFileFactory;
import org.jdtaus.banking.dtaus.Transaction;
import org.jdtaus.editor.Editor;
import org.jdtaus.core.container.ContainerFactory;
import org.jdtaus.core.container.ContainerInitializer;
import org.jdtaus.core.container.ContextFactory;
import org.jdtaus.core.container.ContextInitializer;
import org.jdtaus.core.container.Dependency;
import org.jdtaus.core.container.Implementation;
import org.jdtaus.core.container.ModelFactory;
import org.jdtaus.core.container.Properties;
import org.jdtaus.core.container.Property;
import org.jdtaus.core.container.PropertyException;
import org.jdtaus.core.io.FileOperations;
import org.jdtaus.core.io.util.RandomAccessFileOperations;
import org.jdtaus.core.monitor.spi.Task;
import org.jdtaus.core.monitor.spi.TaskMonitor;
import org.jdtaus.core.text.Message;
import org.jdtaus.editor.GeneratorTask;
import org.jdtaus.editor.PhysicalFileGenerator;
import org.jdtaus.editor.spi.messages.CreatingPhysicalFileMessage;
import org.jdtaus.editor.spi.messages.FileCountConstraintMessage;
import org.jdtaus.editor.spi.messages.IllegalPropertyMessage;
import org.jdtaus.editor.spi.messages.MandatoryPropertyMessage;
import org.jdtaus.editor.spi.messages.TransactionCountConstraintMessage;
import org.jdtaus.editor.spi.messages.UnknownBankleitzahlMessage;

/**
 * {@code PhysicalFileGenerator} reference implementation.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: DefaultPhysicalFileGenerator.java 3516 2007-05-16 10:09:13Z schulte2005 $
 */
public final class DefaultPhysicalFileGenerator
    implements PhysicalFileGenerator, ContainerInitializer
{
    //--Implementation----------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /** Meta-data describing the implementation. */
    private static final Implementation META =
        ModelFactory.getModel().getModules().
        getImplementation(DefaultPhysicalFileGenerator.class.getName());

    //----------------------------------------------------------Implementation--
    //--Constructors------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /**
     * Protected <code>DefaultPhysicalFileGenerator</code> implementation constructor.
     *
     * @param meta Implementation meta-data.
     *
     * @throws NullPointerException if <code>meta</code> is <code>null</code>.
     */
    protected DefaultPhysicalFileGenerator(final Implementation meta)
    {
        super();
        if(meta == null)
        {
            throw new NullPointerException("meta");
        }
        this.initializeProperties(meta.getProperties());
    }
    /**
     * Protected <code>DefaultPhysicalFileGenerator</code> dependency constructor.
     *
     * @param meta dependency meta-data.
     *
     * @throws NullPointerException if <code>meta</code> is <code>null</code>.
     */
    protected DefaultPhysicalFileGenerator(final Dependency meta)
    {
        super();
        if(meta == null)
        {
            throw new NullPointerException("meta");
        }
        this.initializeProperties(meta.getProperties());
    }

    /**
     * Initializes the properties of the instance.
     *
     * @param meta the property values to initialize the instance with.
     *
     * @throws NullPointerException if {@code meta} is {@code null}.
     */
    protected void initializeProperties(final Properties meta)
    {
        Property p;

        if(meta == null)
        {
            throw new NullPointerException("meta");
        }

        p = meta.getProperty("maxDescriptions");
        this._maxDescriptions = ((java.lang.Integer) p.getValue()).intValue();


        p = meta.getProperty("maxTransactions");
        this._maxTransactions = ((java.lang.Integer) p.getValue()).intValue();

    }

    //------------------------------------------------------------Constructors--
    //--ContainerInitializer----------------------------------------------------

    /**
     * Initializes the instance.
     *
     * @see #assertValidProperties()
     */
    public void initialize()
    {
        this.assertValidProperties();
    }

    //----------------------------------------------------ContainerInitializer--
    //--Dependencies------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /** Configured <code>PhysicalFileFactory</code> implementation. */
    private transient PhysicalFileFactory _dependency3;

    /**
     * Gets the configured <code>PhysicalFileFactory</code> implementation.
     *
     * @return the configured <code>PhysicalFileFactory</code> implementation.
     */
    private PhysicalFileFactory getPhysicalFileFactory()
    {
        PhysicalFileFactory ret = null;
        if(this._dependency3 != null)
        {
            ret = this._dependency3;
        }
        else
        {
            ret = (PhysicalFileFactory) ContainerFactory.getContainer().
                getDependency(DefaultPhysicalFileGenerator.class,
                "PhysicalFileFactory");

            if(ModelFactory.getModel().getModules().
                getImplementation(DefaultPhysicalFileGenerator.class.getName()).
                getDependencies().getDependency("PhysicalFileFactory").
                isBound())
            {
                this._dependency3 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>BankleitzahlenVerzeichnis</code> implementation. */
    private transient BankleitzahlenVerzeichnis _dependency2;

    /**
     * Gets the configured <code>BankleitzahlenVerzeichnis</code> implementation.
     *
     * @return the configured <code>BankleitzahlenVerzeichnis</code> implementation.
     */
    private BankleitzahlenVerzeichnis getBankleitzahlenVerzeichnis()
    {
        BankleitzahlenVerzeichnis ret = null;
        if(this._dependency2 != null)
        {
            ret = this._dependency2;
        }
        else
        {
            ret = (BankleitzahlenVerzeichnis) ContainerFactory.getContainer().
                getDependency(DefaultPhysicalFileGenerator.class,
                "BankleitzahlenVerzeichnis");

            if(ModelFactory.getModel().getModules().
                getImplementation(DefaultPhysicalFileGenerator.class.getName()).
                getDependencies().getDependency("BankleitzahlenVerzeichnis").
                isBound())
            {
                this._dependency2 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>TaskMonitor</code> implementation. */
    private transient TaskMonitor _dependency1;

    /**
     * Gets the configured <code>TaskMonitor</code> implementation.
     *
     * @return the configured <code>TaskMonitor</code> implementation.
     */
    private TaskMonitor getTaskMonitor()
    {
        TaskMonitor ret = null;
        if(this._dependency1 != null)
        {
            ret = this._dependency1;
        }
        else
        {
            ret = (TaskMonitor) ContainerFactory.getContainer().
                getDependency(DefaultPhysicalFileGenerator.class,
                "TaskMonitor");

            if(ModelFactory.getModel().getModules().
                getImplementation(DefaultPhysicalFileGenerator.class.getName()).
                getDependencies().getDependency("TaskMonitor").
                isBound())
            {
                this._dependency1 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>Application</code> implementation. */
    private transient Editor _dependency0;

    /**
     * Gets the configured <code>Application</code> implementation.
     *
     * @return the configured <code>Application</code> implementation.
     */
    private Editor getApplication()
    {
        Editor ret = null;
        if(this._dependency0 != null)
        {
            ret = this._dependency0;
        }
        else
        {
            ret = (Editor) ContainerFactory.getContainer().
                getDependency(DefaultPhysicalFileGenerator.class,
                "Application");

            if(ModelFactory.getModel().getModules().
                getImplementation(DefaultPhysicalFileGenerator.class.getName()).
                getDependencies().getDependency("Application").
                isBound())
            {
                this._dependency0 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }

    //------------------------------------------------------------Dependencies--
    //--Properties--------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /**
     * Property {@code maxDescriptions}.
     * @serial
     */
    private int _maxDescriptions;

    /**
     * Gets the value of property <code>maxDescriptions</code>.
     *
     * @return the value of property <code>maxDescriptions</code>.
     */
    protected int getMaxDescriptions()
    {
        return this._maxDescriptions;
    }

    /**
     * Property {@code maxTransactions}.
     * @serial
     */
    private int _maxTransactions;

    /**
     * Gets the value of property <code>maxTransactions</code>.
     *
     * @return the value of property <code>maxTransactions</code>.
     */
    protected int getMaxTransactions()
    {
        return this._maxTransactions;
    }


    //--------------------------------------------------------------Properties--
    //--PhysicalFileGenerator---------------------------------------------------

    public PhysicalFile generate(final File file, final GeneratorTask command)
    throws IOException
    {
        if(file == null)
        {
            throw new NullPointerException("file");
        }
        if(command == null)
        {
            throw new NullPointerException("command");
        }

        this.assertValidGeneratorTask(command);

        final Task task = new Task();
        final Header h = new Header();
        final Transaction t = new Transaction();
        final int[] random = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14};
        final List<AlphaNumericText27> descriptions = new ArrayList(14);
        final FileOperations ops = new RandomAccessFileOperations(
            new RandomAccessFile(file, "rw"));

        h.setAccount(command.getCustomerAccount());
        h.setBank(command.getCustomerBank());
        h.setBankData(null);
        h.setCurrency(Currency.getInstance("EUR"));
        h.setCustomer(command.getCustomerName());
        h.setCreateDate(new Date());
        h.setExecutionDate(new Date());
        t.setAmount(BigInteger.valueOf(1L));
        t.setCurrency(Currency.getInstance("EUR"));
        t.setExecutiveAccount(command.getExecutiveAccount());
        t.setExecutiveBank(command.getExecutiveBank());
        t.setExecutiveExt(command.getExecutiveName());
        t.setExecutiveName(command.getExecutiveName());
        t.setPrimaryBank(null);
        t.setTargetAccount(command.getTargetAccount());
        t.setTargetBank(command.getTargetBank());
        t.setTargetExt(command.getTargetName());
        t.setTargetName(command.getTargetName());

        if(!command.getDescriptionCount().equals(GeneratorTask.RANDOM) &&
            command.getDescriptionCount().intValue() > 0)
        {
            for(int d = command.getDescriptionCount().intValue(); d > 0; d--)
            {
                descriptions.add(command.getExecutiveName());
            }

            t.setDescriptions((AlphaNumericText27[]) descriptions.
                toArray(new AlphaNumericText27[descriptions.size()]));

        }

        int progress = 0;
        int typeIndex = command.getFileTypes().length - 1;
        int keyIndex = command.getTextschluessel().length - 1;
        int randomIndex = random.length - 1;

        task.setIndeterminate(false);
        task.setCancelable(true);
        task.setDescription(new CreatingPhysicalFileMessage());
        task.setMaximum(command.getFileCount().intValue() *
            command.getTransactionCount().intValue() >
            Integer.MAX_VALUE ? Integer.MAX_VALUE :
                (int) (command.getFileCount().intValue() *
            command.getTransactionCount().intValue()));

        task.setMinimum(0);
        task.setProgress(progress);

        ops.setLength(0L);

        try
        {
            this.getTaskMonitor().monitor(task);

            PhysicalFile pFile = getPhysicalFileFactory().getPhysicalFile(ops);

            for(long i = command.getFileCount().intValue(); i > 0L &&
                !task.isCancelled(); i--)
            {
                final LogicalFileType type = command.getFileTypes()[typeIndex];
                if(--typeIndex < 0)
                {
                    typeIndex = command.getFileTypes().length - 1;
                }

                h.setReference(Referenznummer10.valueOf(Long.valueOf(i)));
                h.setType(type);

                if(type.isSendByBank())
                {
                    h.setBankData(command.getExecutiveBank());
                }
                else
                {
                    h.setBankData(null);
                }

                final LogicalFile lFile = pFile.add(h);
                for(int j = command.getTransactionCount().
                    intValue(); j > 0 && !task.isCancelled(); j--)
                {
                    if(++progress > Integer.MAX_VALUE)
                    {
                        progress = Integer.MAX_VALUE;
                    }
                    task.setProgress(progress);

                    t.setReference(Referenznummer11.valueOf(Long.valueOf(j)));

                    Textschluessel txtKey = null;
                    boolean cycle = false;

                    do
                    {
                        txtKey = command.getTextschluessel()[keyIndex];
                        if(--keyIndex < 0)
                        {
                            keyIndex = command.getTextschluessel().length - 1;
                            if(cycle)
                            {
                                txtKey = null;
                                break;
                            }
                            else
                            {
                                cycle = true;
                            }
                        }
                    } while(
                        (type.isDebitAllowed() && !txtKey.isDebit()) ||
                        (type.isRemittanceAllowed() &&
                        !txtKey.isRemittance()));

                    if(txtKey == null)
                    {
                        continue;
                    }

                    t.setType(txtKey);

                    if(command.getDescriptionCount() == GeneratorTask.RANDOM)
                    {
                        final List<AlphaNumericText27> dsc = new ArrayList(14);
                        for(int d = random[randomIndex]; d > 0; d--)
                        {
                            dsc.add(command.getExecutiveName());
                        }

                        if(--randomIndex < 0)
                        {
                            randomIndex = random.length - 1;
                        }

                        t.setDescriptions((AlphaNumericText27[]) dsc.toArray(
                            new AlphaNumericText27[dsc.size()]));

                    }

                    lFile.createTransaction(t);
                }
            }

            if(task.isCancelled())
            {
                pFile = null;
            }

            return pFile;
        }
        catch(IllegalHeaderException e)
        {
            ops.setLength(0L);
            final IllegalGeneratorTaskException ex =
                new IllegalGeneratorTaskException();

            final String[] names = e.getPropertyNames();
            for(int i = names.length - 1; i >= 0; i--)
            {
                final Message[] msgs = e.getMessages(names[i]);
                for(int j = msgs.length - 1; j >= 0; j--)
                {
                    if(names[i].equals(Header.PROP_ACCOUNT))
                    {
                        ex.addMessage(GeneratorTask.PROP_CUSTOMERACCOUNT,
                            msgs[j]);

                    }
                    else if(names[j].equals(Header.PROP_BANK))
                    {
                        ex.addMessage(GeneratorTask.PROP_CUSTOMERBANK,
                            msgs[j]);

                    }
                    else if(names[j].equals(Header.PROP_CUSTOMER))
                    {
                        ex.addMessage(GeneratorTask.PROP_CUSTOMERBANK, msgs[j]);
                    }
                    else if(names[j].equals(Header.PROP_TYPE))
                    {
                        ex.addMessage(GeneratorTask.PROP_FILETYPES, msgs[j]);
                    }
                }
            }

            if(e.getMessages().length > 0)
            {
                final Message[] msgs = e.getMessages();
                for(int i = msgs.length - 1; i >= 0; i--)
                {
                    ex.addMessage(msgs[i]);
                }
            }

            throw ex;
        }
        catch(IllegalTransactionException e)
        {
            ops.setLength(0L);
            final IllegalGeneratorTaskException ex =
                new IllegalGeneratorTaskException();

            final String[] names = e.getPropertyNames();
            for(int i = names.length - 1; i >= 0; i--)
            {
                final Message[] msgs = e.getMessages(names[i]);
                for(int j = msgs.length - 1; j >= 0; j--)
                {
                    if(names[i].equals(Transaction.PROP_EXECUTIVEACCOUNT))
                    {
                        ex.addMessage(GeneratorTask.PROP_EXECUTIVEACCOUNT,
                            msgs[j]);

                    }
                    else if(names[j].equals(Transaction.PROP_EXECUTIVEBANK))
                    {
                        ex.addMessage(GeneratorTask.PROP_EXECUTIVEBANK,
                            msgs[j]);

                    }
                    else if(names[j].equals(Transaction.PROP_EXECUTIVENAME))
                    {
                        ex.addMessage(GeneratorTask.PROP_EXECUTIVENAME,
                            msgs[j]);

                    }
                    else if(names[j].equals(Transaction.PROP_TARGETACCOUNT))
                    {
                        ex.addMessage(GeneratorTask.PROP_TARGETACCOUNT,
                            msgs[j]);

                    }
                    else if(names[j].equals(Transaction.PROP_TARGETBANK))
                    {
                        ex.addMessage(GeneratorTask.PROP_TARGETBANK,
                            msgs[j]);

                    }
                    else if(names[j].equals(Transaction.PROP_TARGETNAME))
                    {
                        ex.addMessage(GeneratorTask.PROP_TARGETNAME, msgs[j]);
                    }
                    else if(names[j].equals(Transaction.PROP_TYPE))
                    {
                        ex.addMessage(GeneratorTask.PROP_TEXTSCHLUESSEL,
                            msgs[j]);

                    }
                }
            }

            if(e.getMessages().length > 0)
            {
                final Message[] msgs = e.getMessages();
                for(int i = msgs.length - 1; i >= 0; i--)
                {
                    ex.addMessage(msgs[i]);
                }
            }

            throw ex;
        }
        catch(PhysicalFileException e)
        {
            throw new AssertionError(e);
        }
        finally
        {
            if(task.isCancelled())
            {
                new RandomAccessFile(file ,"rw").setLength(0L);
            }

            this.getTaskMonitor().finish(task);
        }
    }

    //---------------------------------------------------PhysicalFileGenerator--
    //--DefaultPhysicalFileGenerator--------------------------------------------

    /**
     * Checks configured properties.
     *
     * @throws PropertyException for illegal property values.
     */
    protected void assertValidProperties()
    {
        if(this.getMaxDescriptions() < 0)
        {
            throw new PropertyException("maxDescription",
                Integer.valueOf(this.getMaxDescriptions()));

        }

        if(this.getMaxTransactions() < 0)
        {
            throw new PropertyException("maxTransactions",
                Integer.valueOf(this.getMaxTransactions()));

        }
    }

    /**
     * Checks a given {@code GeneratorTask} to hold valid values.
     *
     * @param command the instance to check.
     *
     * @throws NullPointerException if {@code command} is {@code null}.
     * @throws IllegalGeneratorTaskException if {@code command} holds illegal
     * values.
     */
    protected void assertValidGeneratorTask(final GeneratorTask command)
    {
        if(command == null)
        {
            throw new NullPointerException("command");
        }

        final IllegalGeneratorTaskException e =
            new IllegalGeneratorTaskException();

        if(command.getCustomerAccount() == null)
        {
            e.addMessage(GeneratorTask.PROP_CUSTOMERACCOUNT,
                new MandatoryPropertyMessage());

        }

        if(command.getCustomerBank() == null)
        {
            e.addMessage(GeneratorTask.PROP_CUSTOMERBANK,
                new MandatoryPropertyMessage());

        }
        else if(!this.checkBankleitzahl(command.getCustomerBank()))
        {
            e.addMessage(GeneratorTask.PROP_CUSTOMERBANK,
                new UnknownBankleitzahlMessage());

        }

        if(command.getCustomerName() == null)
        {
            e.addMessage(GeneratorTask.PROP_CUSTOMERNAME,
                new MandatoryPropertyMessage());

        }
        if(command.getExecutiveAccount() == null)
        {
            e.addMessage(GeneratorTask.PROP_EXECUTIVEACCOUNT,
                new MandatoryPropertyMessage());

        }

        if(command.getExecutiveBank() == null)
        {
            e.addMessage(GeneratorTask.PROP_EXECUTIVEBANK,
                new MandatoryPropertyMessage());

        }
        else if(!this.checkBankleitzahl(command.getExecutiveBank()))
        {
            e.addMessage(GeneratorTask.PROP_EXECUTIVEBANK,
                new UnknownBankleitzahlMessage());

        }

        if(command.getExecutiveName() == null)
        {
            e.addMessage(GeneratorTask.PROP_EXECUTIVENAME,
                new MandatoryPropertyMessage());

        }
        if(command.getTargetAccount() == null)
        {
            e.addMessage(GeneratorTask.PROP_TARGETACCOUNT,
                new MandatoryPropertyMessage());

        }

        if(command.getTargetBank() == null)
        {
            e.addMessage(GeneratorTask.PROP_TARGETBANK,
                new MandatoryPropertyMessage());

        }
        else if(!this.checkBankleitzahl(command.getTargetBank()))
        {
            e.addMessage(GeneratorTask.PROP_TARGETBANK,
                new UnknownBankleitzahlMessage());

        }

        if(command.getTargetName() == null)
        {
            e.addMessage(GeneratorTask.PROP_TARGETNAME,
                new MandatoryPropertyMessage());

        }
        if(command.getFileTypes() == null)
        {
            e.addMessage(GeneratorTask.PROP_FILETYPES,
                new MandatoryPropertyMessage());

        }
        if(command.getTextschluessel() == null)
        {
            e.addMessage(GeneratorTask.PROP_TEXTSCHLUESSEL,
                new MandatoryPropertyMessage());

        }
        if(command.getFileCount() == null)
        {
            e.addMessage(GeneratorTask.PROP_FILECOUNT,
                new MandatoryPropertyMessage());

        }
        else if(command.getFileCount().intValue() < 0)
        {
            e.addMessage(GeneratorTask.PROP_FILECOUNT,
                new IllegalPropertyMessage());

        }

        if(command.getTransactionCount() == null)
        {
            e.addMessage(GeneratorTask.PROP_TRANSACTIONCOUNT,
                new MandatoryPropertyMessage());

        }
        else if(command.getTransactionCount().intValue() < 0 ||
            command.getTransactionCount().intValue() >
            this.getMaxTransactions())
        {
            e.addMessage(GeneratorTask.PROP_TRANSACTIONCOUNT,
                new IllegalPropertyMessage());

        }

        if(command.getDescriptionCount() == null)
        {
            e.addMessage(GeneratorTask.PROP_DESCRIPTIONCOUNT,
                new MandatoryPropertyMessage());

        }
        else if(!command.getDescriptionCount().equals(GeneratorTask.RANDOM) &&
            command.getDescriptionCount().intValue() < 0 ||
            command.getDescriptionCount().intValue() >
            this.getMaxDescriptions())
        {
            e.addMessage(GeneratorTask.PROP_DESCRIPTIONCOUNT,
                new IllegalPropertyMessage());

        }

        if(command.getFileTypes() != null &&
            command.getFileCount() != null &&
            command.getFileCount().intValue() > 0 &&
            command.getFileTypes().length == 0)
        {
            e.addMessage(new FileCountConstraintMessage(
                command.getFileCount().intValue(),
                command.getFileTypes().length));

        }

        if(command.getTextschluessel() != null &&
            command.getTransactionCount() != null &&
            command.getTransactionCount().intValue() > 0 &&
            command.getTextschluessel().length == 0)
        {
            e.addMessage(
                new TransactionCountConstraintMessage(
                command.getTransactionCount().intValue(),
                command.getTextschluessel().length));

        }

        if(e.getMessages().length > 0)
        {
            throw e;
        }
    }

    /**
     * Checks a given Bankleitzahl for existence in the
     * {@code BankleitzahlenVerzeichnis}.
     *
     * @param code the Bankleitzahl to check.
     *
     * @throws NullPointerException if {@code code} is {@code null}.
     */
    private boolean checkBankleitzahl(final Bankleitzahl code)
    {
        if(code == null)
        {
            throw new NullPointerException("code");
        }

        boolean valid = true;
        if(!this.getApplication().getPreferences().
            isBankcodeValidationDisabled())
        {
            valid = this.getBankleitzahlenVerzeichnis().
                getHeadOffice(code) != null;

        }

        return valid;
    }

    //--------------------------------------------DefaultPhysicalFileGenerator--
}
