/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package org.jdtaus.editor.spi;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import org.jdtaus.core.text.Message;
import org.jdtaus.core.text.Messages;
import org.jdtaus.editor.GeneratorTask;

/**
 * Gets thrown whenever an illegal generator task is passed to a method
 * expecting a legal generator task.
 * <p>Example: Throwing an {@code IllegalGeneratorTaskException}<br/><blockquote><pre>
 * IllegalGeneratorTaskException e = new IllegalGeneratorTaskException();
 * e.addMessage(message);
 * e.addMessage(GeneratorTask.PROP_<i>xyz</i>, message);
 * throw e;</pre></blockquote></p>
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: IllegalGeneratorTaskException.java 3516 2007-05-16 10:09:13Z schulte2005 $
 */
public class IllegalGeneratorTaskException
    extends org.jdtaus.editor.IllegalGeneratorTaskException
{
    //--Constructors------------------------------------------------------------

    /** Creates a new {@code IllegalHeaderException} instance. */
    public IllegalGeneratorTaskException()
    {
        super();
    }

    //------------------------------------------------------------Constructors--
    //--IllegalGeneratorTaskException-------------------------------------------

    /** Key to the list of messages not bound to any particular property. */
    private static final String PROP_UNSPECIFIED =
        GeneratorTask.class.getName();

    /**
     * The messages describing the exception.
     * @serial
     */
    private Map messages = new HashMap(100);

    /**
     * Adds a message to the instance.
     *
     * @param message the message to add to the instance.
     *
     * @throws NullPointerException if {@code message} is {@code null}.
     */
    public void addMessage(final Message message)
    {
        this.addMessage(PROP_UNSPECIFIED, message);
    }

    /**
     * Adds messages to the instance.
     *
     * @param messages the messages to add to the instance.
     *
     * @throws NullPointerException if {@code messages} is {@code null}.
     */
    public final void addMessages(final Messages messages)
    {
        if(messages == null)
        {
            throw new NullPointerException("messages");
        }

        for(int i = messages.size() - 1; i >= 0; i--)
        {
            this.addMessage(messages.getMessage(i));
        }
    }

    /**
     * Adds a message bound to a property to the instance.
     *
     * @param propertyName the name of a property {@code message} is bound to.
     * @param message the message to add to the instance.
     *
     * @throws NullPointerException if either {@code message} or
     * {@code propertyName} is {@code null}.
     */
    public void addMessage(final String propertyName, final Message message)
    {
        if(propertyName == null)
        {
            throw new NullPointerException("propertyName");
        }
        if(message == null)
        {
            throw new NullPointerException("message");
        }

        List msgs = (List) this.messages.get(propertyName);
        if(msgs == null)
        {
            msgs = new LinkedList();
            this.messages.put(propertyName, msgs);
        }

        msgs.add(message);
    }

    /**
     * Adds messages bound to a property to the instance.
     *
     * @param propertyName the name of a property {@code messages} are bound to.
     * @param messages the messages to add to the instance.
     *
     * @throws NullPointerException if either {@code messages} or
     * {@code propertyName} is {@code null}.
     */
    public final void addMessages(final String propertyName,
        final Messages messages)
    {
        if(propertyName == null)
        {
            throw new NullPointerException("propertyName");
        }
        if(messages == null)
        {
            throw new NullPointerException("messages");
        }

        for(int i = messages.size() - 1; i >= 0; i--)
        {
            this.addMessage(propertyName, messages.getMessage(i));
        }
    }

    /**
     * Gets all messages describing the exception.
     *
     * @return an array of messages describing the exception or an empty array
     * if the instance does not hold any messages.
     */
    public Message[] getMessages()
    {
        final List col = new LinkedList();
        for(Iterator it = this.messages.keySet().iterator(); it.hasNext();)
        {
            final String propertyName = (String) it.next();
            col.addAll((List) this.messages.get(propertyName));
        }

        return (Message[]) col.toArray(new Message[col.size()]);
    }

    /**
     * Gets messages bound to a property removing these messages from the
     * instance.
     *
     * @param propertyName the name of a property to return any messages for.
     *
     * @return all messages bound to a property with name {@code propertyName}
     * or an empty array if the instance does not hold messages for a property
     * with name {@code propertyName}.
     *
     * @throws NullPointerException if {@code propertyName} is {@code null}.
     */
    public Message[] getMessages(final String propertyName)
    {
        if(propertyName == null)
        {
            throw new NullPointerException("propertyName");
        }

        final List msgs = (List) this.messages.remove(propertyName);
        return msgs == null ? new Message[0] :
            (Message[]) msgs.toArray(new Message[msgs.size()]);

    }

    /**
     * Gets the names of all properties for which the exception holds messages.
     *
     * @return an array of the names of all properties for which the exception
     * holds messages or an empty array if the exception does not hold any
     * message bound to a property.
     */
    public String[] getPropertyNames()
    {
        final List names = new ArrayList(this.messages.size());
        for(Iterator it = this.messages.keySet().iterator(); it.hasNext();)
        {
            final String name = (String) it.next();
            if(!PROP_UNSPECIFIED.equals(name))
            {
                names.add(name);
            }
        }

        return (String[]) names.toArray(new String[names.size()]);
    }

    /**
     * Creates a string representing the messages of the instance.
     *
     * @return a string representing the messages of the instance.
     */
    private String internalString()
    {
        final StringBuffer buf = new StringBuffer(200);
        final String[] propertyNames = this.getPropertyNames();
        for(int i = 0; i < propertyNames.length; i++)
        {
            buf.append(propertyNames[i]);

            final Message[] msgs = this.getMessages(propertyNames[i]);
            for(int j = 0; j < msgs.length; j++)
            {
                buf.append('\t').append("\n\tmessage[").append(j).append("]=").
                    append(msgs[j].getText(Locale.getDefault()));

            }

            buf.append('\n');
        }

        final Message[] msgs = this.getMessages();
        if(msgs.length > 0)
        {
            buf.append(PROP_UNSPECIFIED).append('\n');
        }

        for(int i = 0; i < msgs.length; i++)
        {
            buf.append("\n\tmessage[").append(i).append("]=").
                append(msgs[i].getText(Locale.getDefault()));

        }

        return buf.toString();
    }

    //--------------------------------------------------IllegalHeaderException--
    //--Object------------------------------------------------------------------

    /**
     * Returns a string representation of the object.
     *
     * @return a string representation of the object.
     */
    public String toString()
    {
        return super.toString() + '\n' + this.internalString();
    }

    //------------------------------------------------------------------Object--
}
